"""
   Copyright 2016 University of Auckland

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
"""
from opencmiss.argon.argonsceneviewer import ArgonSceneviewer

LAYOUT1 = {
    "Name": "Layout1",
    "Scenes": [
        {
            "Row": 0,
            "Col": 0,
            "Sceneviewer": {},
        }
    ]
}

LAYOUT2x2GRID = {
    "Name":"Layout2x2Grid",
    "GridSpecification":{
       "NumRows":2,
       "NumCols":2,
       "Left":None,
       "Bottom":None,
       "Right":None,
       "Top":None,
       "WidthSpace":None,
       "HeightSpace":None,
       "WidthRatios":None,
       "HeightRatios":None
    },
    "Scenes":[
       {
          "Row":1,
          "Col":1,
          "Sceneviewer":{},
       },
       {
          "Row":0,
          "Col":1,
          "Sceneviewer":{},
       },
       {
          "Row":1,
          "Col":0,
          "Sceneviewer":{},
       },
       {
          "Row":0,
          "Col":0,
          "Sceneviewer":{},
       }
    ]
 }

LAYOUTS = [
    LAYOUT1,
    LAYOUT2x2GRID,
]


class ArgonViewManager(object):
    """
    Manages and serializes ArgonViews.
    """

    def __init__(self, zincContext):
        self._zincContext = zincContext
        self._views = []
        self._activeView = None

    def getZincContext(self):
        return self._zincContext

    def deserialize(self, d):
        self._activeView = d["ActiveView"] if "ActiveView" in d else None
        if "Children" in d:
            for i in d["Children"]:
                view = ArgonView(self._zincContext)
                view.deserialize(i)
                self._views.append(view)
        
    def serialize(self):
        dictOutput = {}
        if self._activeView:
            dictOutput["ActiveView"] = self._activeView
        dictOutput["Children"] = []
        if self._views:
            tmpOutput = []
            for view in self._views:
                tmpOutput.append(view.serialize())
            dictOutput["Children"] = tmpOutput
        return dictOutput

    def getActiveView(self):
        return self._activeView

    def setActiveView(self, view):
        self._activeView = view

    def getView(self, index):
        return self._views[index]

    def getViews(self):
        return self._views

    def setViews(self, views):
        self._views = views

    def addView(self, view_type, name=None):
        for layout in LAYOUTS:
            if layout["Name"] == view_type:
                new_view = ArgonView(self._zincContext)
                new_view.deserialize(layout)
                if name is None:
                    name = view_type
                if self._name_in_use(name):
                    name = self._next_available_name(name)
                new_view.setName(name)
                self._views.append(new_view)
                break

    def removeView(self, identifier):
        del self._views[identifier]

    def _name_in_use(self, name):
        for view in self._views:
            if view.getName() == name:
                return True

        return False

    def _next_available_name(self, name_stem):
        next_name = name_stem
        iteration = 1
        while self._name_in_use(f"{name_stem}_{iteration}"):
            iteration += 1

        return f"{name_stem}_{iteration}"

    def updateSceneviewers(self, view_index, sceneviewers_info):
        if 0 <= view_index < len(self._views):
            view = self._views[view_index]
            for sceneviewer_info in sceneviewers_info:
                view.updateSceneviewer(sceneviewer_info["Row"], sceneviewer_info["Col"], sceneviewer_info["Sceneviewer"])


class ArgonView(object):
    """
    Defines and serializes a single view.
    """

    def __init__(self, zincContext):
        self._zincContext = zincContext
        self._name = None
        self._gridSpecification = None
        self._scenes = []

    def getZincContext(self):
        return self._zincContext

    def deserialize(self, d):
        self._name = d["Name"] if "Name" in d else None
        self._gridSpecification = d["GridSpecification"] if "GridSpecification" in d else None

        if "Scenes" in d:
            for s in d["Scenes"]:
                scene = {"Row": 0, "Col": 0}
                if "Row" in s:
                    scene["Row"] = s["Row"]
                if "Col" in s:
                    scene["Col"] = s["Col"]
                if "Sceneviewer" in s:
                    sceneviewer = ArgonSceneviewer(self._zincContext)
                    sceneviewer.deserialize(s["Sceneviewer"])
                    scene["Sceneviewer"] = sceneviewer

                self._scenes.append(scene)
        
    def serialize(self):
        dictOutput = {}
        if self._name:
            dictOutput["Name"] = self._name
        if self._gridSpecification:
            dictOutput["GridSpecification"] = self._gridSpecification
        dictOutput["Scenes"] = []
        if self._scenes:
            for scene in self._scenes:
                tmp_output = {
                    "Row": scene["Row"],
                    "Col": scene["Col"],
                    "Sceneviewer": scene["Sceneviewer"].serialize(),
                }
                dictOutput["Scenes"].append(tmp_output)
        return dictOutput

    def getName(self):
        return self._name

    def setName(self, name):
        self._name = name

    def getScenes(self):
        return self._scenes

    def getGridSpecification(self):
        return self._gridSpecification

    def updateSceneviewer(self, row, col, sceneviewer):
        for scene in self._scenes:
            if scene["Row"] == row and scene["Col"] == col:
                scene["Sceneviewer"].updateParameters(sceneviewer)



"""
   Copyright 2016 University of Auckland

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
"""


class ArgonError(Exception):

    def __init__(self, message):
        self.message = message

    def __str__(self):
        return repr(self.message)

    def getMessage(self):
        return self.message



"""
   Copyright 2015 University of Auckland

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
"""
from opencmiss.zinc.sceneviewer import Sceneviewer


SceneviewerProjectionModeMap = {
    Sceneviewer.PROJECTION_MODE_PARALLEL: "PARALLEL",
    Sceneviewer.PROJECTION_MODE_PERSPECTIVE: "PERSPECTIVE"
}


def SceneviewerProjectionModeEnumFromString(projectionModeName: str):
    for mode, name in SceneviewerProjectionModeMap.items():
        if name == projectionModeName:
            return mode
    return Sceneviewer.PROJECTION_MODE_INVALID


def SceneviewerProjectionModeEnumToString(projectionMode):
    for mode, name in SceneviewerProjectionModeMap.items():
        if mode == projectionMode:
            return name
    return None


SceneviewerTransparencyModeMap = {
    Sceneviewer.TRANSPARENCY_MODE_FAST: "FAST",
    Sceneviewer.TRANSPARENCY_MODE_SLOW: "SLOW",
    Sceneviewer.TRANSPARENCY_MODE_ORDER_INDEPENDENT: "ORDER_INDEPENDENT"
}


def SceneviewerTransparencyModeEnumFromString(transparencyModeName: str):
    for mode, name in SceneviewerTransparencyModeMap.items():
        if name == transparencyModeName:
            return mode
    return Sceneviewer.TRANSPARENCY_MODE_INVALID


def SceneviewerTransparencyModeEnumToString(transparencyMode):
    for mode, name in SceneviewerTransparencyModeMap.items():
        if mode == transparencyMode:
            return name
    return None


class ArgonSceneviewer(object):

    def __init__(self, zinc_context, parent=None):
        self._zinc_context = zinc_context
        self._parent = parent
        self._anti_alias_sampling = default_anti_alias_sampling
        self._background_colour_RGB = default_background_colour_RGB
        self._eye_position = default_eye_position
        self._far_clipping_plane = default_far_clipping_plane
        self._lighting_local_viewer = default_lighting_local_viewer
        self._lighting_two_sided = default_lighting_two_sided
        self._lookat_position = default_lookat_position
        self._near_clipping_plane = default_near_clipping_plane
        self._perturb_lines_flag = default_perturb_lines_flag
        self._projection_mode = default_projection_mode
        self._scene = default_scene
        self._scene_filter = default_scene_filter
        self._translation_rate = default_translation_rate
        self._transparency_mode = default_transparency_mode
        self._transparency_layers = default_transparency_layers
        self._tumble_rate = default_tumble_rate
        self._up_vector = default_up_vector
        self._view_angle = default_view_angle
        self._zoom_rate = default_zoom_rate

    def applyParameters(self, sceneviewer):
        sceneviewer.setEyePosition(self._eye_position)
        sceneviewer.setLookatPosition(self._lookat_position)
        sceneviewer.setUpVector(self._up_vector)
        sceneviewer.setNearClippingPlane(self._near_clipping_plane)
        sceneviewer.setFarClippingPlane(self._far_clipping_plane)
        sceneviewer.setBackgroundColourRGB(self._background_colour_RGB)
        sceneviewer.setAntialiasSampling(self._anti_alias_sampling)
        sceneviewer.setProjectionMode(self._projection_mode)
        sceneviewer.setTransparencyMode(self._transparency_mode)
        sceneviewer.setTransparencyLayers(self._transparency_layers)
        sceneviewer.setLightingTwoSided(self._lighting_two_sided)
        sceneviewer.setPerturbLinesFlag(self._perturb_lines_flag)
        sceneviewer.setViewAngle(self._view_angle)
        root_region = self._zinc_context.getDefaultRegion()
        if self._scene is not None:
            scene_region = root_region.findChildByName(self._scene)
            if scene_region.isValid():
                sceneviewer.setScene(scene_region.getScene())

    def updateParameters(self, sceneviewer):
        result, eye = sceneviewer.getEyePosition()
        result, lookat = sceneviewer.getLookatPosition()
        result, up_vector = sceneviewer.getUpVector()
        near = sceneviewer.getNearClippingPlane()
        far = sceneviewer.getFarClippingPlane()
        result, colourRGB = sceneviewer.getBackgroundColourRGB()
        antialiasValue = sceneviewer.getAntialiasSampling()
        projection_mode = sceneviewer.getProjectionMode()
        transparency_mode = sceneviewer.getTransparencyMode()
        transparency_layers = sceneviewer.getTransparencyLayers()
        flag_is_two_sided = sceneviewer.isLightingTwoSided()
        flag_perturb_lines = sceneviewer.getPerturbLinesFlag()
        view_angle = sceneviewer.getViewAngle()
        scene_region = sceneviewer.getScene().getRegion().getName()

        self._anti_alias_sampling = antialiasValue
        self._background_colour_RGB = colourRGB
        self._eye_position = eye
        self._far_clipping_plane = far
        self._lighting_local_viewer = default_lighting_local_viewer
        self._lighting_two_sided = flag_is_two_sided
        self._lookat_position = lookat
        self._near_clipping_plane = near
        self._perturb_lines_flag = flag_perturb_lines
        self._projection_mode = projection_mode
        self._scene = scene_region
        self._scene_filter = default_scene_filter
        self._translation_rate = default_translation_rate
        self._transparency_mode = transparency_mode
        self._transparency_layers = transparency_layers
        self._tumble_rate = default_tumble_rate
        self._up_vector = up_vector
        self._view_angle = view_angle
        self._zoom_rate = default_zoom_rate

    def get_view_parameters(self):
        return {'farClippingPlane': self._far_clipping_plane, 'nearClippingPlane': self._near_clipping_plane,
                'eyePosition': self._eye_position, 'lookAtPosition': self._lookat_position, 'upVector': self._up_vector,
                'viewAngle': self._view_angle}

    def deserialize(self, d):
        # d = json.loads(s)
        self._anti_alias_sampling = d["AntialiasSampling"] if "AntialiasSampling" in d else default_anti_alias_sampling
        self._background_colour_RGB = d["BackgroundColourRGB"] if "BackgroundColourRGB" in d \
            else default_background_colour_RGB
        self._eye_position = d["EyePosition"] if "EyePosition" in d else default_eye_position
        self._far_clipping_plane = d["FarClippingPlane"] if "FarClippingPlane" in d else default_far_clipping_plane
        self._lighting_local_viewer = d["LightingLocalViewer"] if "LightingLocalViewer" in d \
            else default_lighting_local_viewer
        self._lighting_two_sided = d["LightingTwoSided"] if "LightingTwoSided" in d else default_lighting_two_sided
        self._lookat_position = d["LookatPosition"] if "LookatPosition" in d else default_lookat_position
        self._near_clipping_plane = d["NearClippingPlane"] if "NearClippingPlane" in d else default_near_clipping_plane
        self._perturb_lines_flag = d["PerturbLinesFlag"] if "PerturbLinesFlag" in d else default_perturb_lines_flag
        self._projection_mode = SceneviewerProjectionModeEnumFromString(d["ProjectionMode"])  if "ProjectionMode" in d \
            else default_projection_mode
        self._scene = d["Scene"] if "Scene" in d else default_scene
        self._scene_filter = d["Scenefilter"] if "Scenefilter" in d else default_scene_filter
        self._translation_rate = d["TranslationRate"] if "TranslationRate" in d else default_translation_rate
        self._transparency_mode = SceneviewerTransparencyModeEnumFromString(d["TransparencyMode"]) \
            if "TransparencyMode" in d else default_transparency_mode
        self._transparency_layers = d["TransparencyLayers"] if "TransparencyLayers" in d \
            else default_transparency_layers
        self._tumble_rate = d["TumbleRate"] if "TumbleRate" in d else default_tumble_rate
        self._up_vector = d["UpVector"] if "UpVector" in d else default_up_vector
        self._view_angle = d["ViewAngle"] if "ViewAngle" in d else default_view_angle
        self._zoom_rate = d["ZoomRate"] if "ZoomRate" in d else default_zoom_rate

    def serialize(self):
        d = {}
        d["AntialiasSampling"] = self._anti_alias_sampling
        d["BackgroundColourRGB"] = self._background_colour_RGB
        d["EyePosition"] = self._eye_position
        d["FarClippingPlane"] = self._far_clipping_plane
        d["LightingLocalViewer"] = self._lighting_local_viewer
        d["LightingTwoSided"] = self._lighting_two_sided
        d["LookatPosition"] = self._lookat_position
        d["NearClippingPlane"] = self._near_clipping_plane
        d["PerturbLinesFlag"] = self._perturb_lines_flag
        d["ProjectionMode"] = SceneviewerProjectionModeEnumToString(self._projection_mode)
        d["Scene"] = self._scene
        d["Scenefilter"] = self._scene_filter
        d["TranslationRate"] = self._translation_rate
        d["TransparencyMode"] = SceneviewerTransparencyModeEnumToString(self._transparency_mode)
        d["TransparencyLayers"] = self._transparency_layers
        d["TumbleRate"] = self._tumble_rate
        d["UpVector"] = self._up_vector
        d["ViewAngle"] = self._view_angle
        d["ZoomRate"] = self._zoom_rate
        return d


default_anti_alias_sampling = 0
default_background_colour_RGB = [1, 1, 1]
default_eye_position = [0, 0, 3.88551982890656]
default_far_clipping_plane = 7.88551982890656
default_lighting_local_viewer = False
default_lighting_two_sided = True
default_lookat_position = [0, 0, 0]
default_near_clipping_plane = 0.1942759914453282
default_perturb_lines_flag = False
default_projection_mode = Sceneviewer.PROJECTION_MODE_PERSPECTIVE
default_scene = "/"
default_scene_filter = "default"
default_translation_rate = 1
default_transparency_mode = Sceneviewer.TRANSPARENCY_MODE_FAST
default_transparency_layers = 1
default_tumble_rate = 1.5
default_up_vector = [0, 1, 0]
default_view_angle = 0.6981317007977318
default_zoom_rate = 1



"""
   Copyright 2016 University of Auckland

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
"""
import json

from opencmiss.zinc.status import OK as ZINC_OK
from opencmiss.argon.argonerror import ArgonError

SPECTRUM_GLYPH_NAME_FORMAT = 'colour_bar_{0}'


class ArgonSpectrums(object):
    """
    Manages and serializes Zinc Spectrums within Neon.
    Generates colour bar glyphs for spectrums, which is automatically done if if not found on loading.
    """

    def __init__(self, zincContext):
        self._zincContext = zincContext
        self._spectrummodule = zincContext.getSpectrummodule()
        self._findOrCreateAllSpectrumGlyphColourBars()

    def getZincContext(self):
        return self._zincContext

    def deserialize(self, dictInput):
        spectrumsDescription = json.dumps(dictInput)
        result = self._spectrummodule.readDescription(spectrumsDescription)
        if result != ZINC_OK:
            raise ArgonError("Failed to read spectrums")
        self._findOrCreateAllSpectrumGlyphColourBars()

    def serialize(self):
        spectrumsDescription = self._spectrummodule.writeDescription()
        dictOutput = json.loads(spectrumsDescription)
        return dictOutput

    def _findOrCreateAllSpectrumGlyphColourBars(self):
        """
        Ensures there exists a colour bar for each spectrum.
        """
        iterator = self._spectrummodule.createSpectrumiterator()
        spectrum = iterator.next()
        while spectrum.isValid():
            self.findOrCreateSpectrumGlyphColourBar(spectrum)
            spectrum = iterator.next()

    def _findSpectrumGlyphColourBar(self, spectrum):
        """
        Find or create a GlyphColourBar for spectrum in the glyph module.
        Newly created colour bar is set up for display in the normalised window coordinates at left.
        """
        glyphmodule = self._zincContext.getGlyphmodule()
        glyphName = SPECTRUM_GLYPH_NAME_FORMAT.format(spectrum.getName())

        # try to find by name first
        glyph = glyphmodule.findGlyphByName(glyphName)
        colourBar = glyph.castColourBar()
        if colourBar.isValid() and (colourBar.getSpectrum() == spectrum):
            return colourBar

        # attempt to find by matching spectrum but do not rename it
        glyphiterator = glyphmodule.createGlyphiterator()
        glyph = glyphiterator.next()
        while glyph.isValid():
            colourBar = glyph.castColourBar()
            if colourBar.isValid() and (colourBar.getSpectrum() == spectrum):
                return colourBar
            glyph = glyphiterator.next()

        return None

    def findOrCreateSpectrumGlyphColourBar(self, spectrum):
        """
        Find or create a GlyphColourBar for spectrum in the glyph module.
        Newly created colour bar is set up for display in the normalised window coordinates at left.
        """
        colourBar = self._findSpectrumGlyphColourBar(spectrum)
        if colourBar:
            return colourBar

        glyphmodule = self._zincContext.getGlyphmodule()
        glyphName = SPECTRUM_GLYPH_NAME_FORMAT.format(spectrum.getName())

        # create a new colour bar, matching Cmgui's defaults:
        glyphmodule.beginChange()
        colourBar = glyphmodule.createGlyphColourBar(spectrum)
        tmpName = glyphName
        i = 1
        while (colourBar.setName(tmpName) != ZINC_OK):
            tmpName = glyphName + str(i)
            i += 1
        colourBar.setManaged(True)
        colourBar.setCentre([-0.9, 0.0, 0.5])
        colourBar.setAxis([0.0, 1.6, 0.0])  # includes length
        colourBar.setSideAxis([0.06, 0.0, 0.0])  # includes radius
        colourBar.setExtendLength(0.06)
        colourBar.setTickLength(0.04)
        colourBar.setLabelDivisions(10)
        colourBar.setNumberFormat('%+.4e')
        glyphmodule.endChange()
        return colourBar

    def renameSpectrum(self, spectrum, name):
        """
        Renames spectrum and its glyph
        :return True on success, otherwise False (means name not set)
        """
        colourBar = self.findOrCreateSpectrumGlyphColourBar(spectrum)
        result = spectrum.setName(name)
        if result == ZINC_OK:
            glyphName = SPECTRUM_GLYPH_NAME_FORMAT.format(name)
            tmpName = glyphName
            i = 1
            while (colourBar.setName(tmpName) != ZINC_OK):
                tmpName = glyphName + str(i)
                i += 1
            return True
        return False

    def removeSpectrumByName(self, name):
        """
        Unmanages spectrum and its colour bar. Note spectrum is only removed if neither are in use.
        :return True if spectrum and colour bar removed, false if failed i.e. either are in use.
        """
        spectrum = self._spectrummodule.findSpectrumByName(name)
        colourBar = self._findSpectrumGlyphColourBar(spectrum)
        if colourBar:
            colourBarName = colourBar.getName()
            colourBar.setManaged(False)
            del colourBar
            colourBar = self._findSpectrumGlyphColourBar(spectrum)
            if colourBar and (colourBar.getName() == colourBarName):
                # colour bar is in use; do not remove spectrum
                colourBar.setManaged(True)
                return False
        spectrum.setManaged(False)
        del spectrum
        spectrum = self._spectrummodule.findSpectrumByName(name)
        if spectrum.isValid():
            # spectrum is in use so can't remove
            spectrum.setManaged(True)
            self.findOrCreateSpectrumGlyphColourBar(spectrum)
            return False
        return True



"""
   Copyright 2015 University of Auckland

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
"""
import json

from packaging import version

from opencmiss.argon import __version__
from opencmiss.argon.argonregion import ArgonRegion, REGION_PATH_SEPARATOR
from opencmiss.argon.argonspectrums import ArgonSpectrums
from opencmiss.argon.argonmaterials import ArgonMaterials
from opencmiss.argon.argonviews import ArgonViewManager
from opencmiss.argon.argontessellations import ArgonTessellations
from opencmiss.argon.argonerror import ArgonError
from opencmiss.argon.argonlogger import ArgonLogger
from opencmiss.argon.settings import mainsettings

from opencmiss.zinc.context import Context
from opencmiss.zinc.material import Material


class ArgonDocument(object):

    def __init__(self, name="Argon"):
        self._name = name
        self._zincContext = None
        self._rootRegion = None
        self._spectrums = None
        self._materials = None
        self._view_manager = None
        self._tessellations = None

    def checkVersion(self, minimum_required):
        if version.parse(__version__) < version.parse(minimum_required):
            raise SyntaxError(f"Argon document error - Argon document must be at least version '{minimum_required}'.")

    def getName(self):
        return self._zincContext.getName()

    def initialiseVisualisationContents(self):
        self._zincContext = Context(self._name)

        sceneviewermodule = self._zincContext.getSceneviewermodule()
        sceneviewermodule.setDefaultBackgroundColourRGB([1.0, 1.0, 1.0])

        # set up standard materials and glyphs
        materialmodule = self._zincContext.getMaterialmodule()
        materialmodule.beginChange()
        materialmodule.defineStandardMaterials()
        # make default material black
        defaultMaterial = materialmodule.getDefaultMaterial()
        defaultMaterial.setAttributeReal3(Material.ATTRIBUTE_AMBIENT, [0.0, 0.0, 0.0])
        defaultMaterial.setAttributeReal3(Material.ATTRIBUTE_DIFFUSE, [0.0, 0.0, 0.0])
        # still want surfaces to default to white material
        white = materialmodule.findMaterialByName("white")
        materialmodule.setDefaultSurfaceMaterial(white)
        materialmodule.endChange()
        glyphmodule = self._zincContext.getGlyphmodule()
        glyphmodule.defineStandardGlyphs()

        zincRootRegion = self._zincContext.getDefaultRegion()
        self._rootRegion = ArgonRegion(name=None, zincRegion=zincRootRegion, parent=None)
        self._rootRegion.connectRegionChange(self._regionChange)

        self._materials = materialmodule
        self._spectrums = ArgonSpectrums(self._zincContext)
        self._materials = ArgonMaterials(self._zincContext)
        self._view_manager = ArgonViewManager(self._zincContext)
        self._tessellations = ArgonTessellations(self._zincContext)
        ArgonLogger.setZincContext(self._zincContext)

    def freeVisualisationContents(self):
        """
        Deletes subobjects of document to help free memory held by Zinc objects earlier.
        """
        self._rootRegion.freeContents()
        del self._tessellations
        del self._spectrums
        del self._materials
        del self._view_manager
        del self._rootRegion
        del self._zincContext

    def _regionChange(self, changedRegion, treeChange):
        """
        If root region has changed, set its new Zinc region as Zinc context's default region.
        :param changedRegion: The top region changed
        :param treeChange: True if structure of tree, or zinc objects reconstructed
        """
        if treeChange and (changedRegion is self._rootRegion):
            zincRootRegion = changedRegion.getZincRegion()
            self._zincContext.setDefaultRegion(zincRootRegion)

    def deserialize(self, state):
        """
        :param  state: string serialisation of Argon JSON document
        """
        d = json.loads(state)
        if not (("OpenCMISS-Argon Version" in d) and ("RootRegion" in d)):
            raise ArgonError("Invalid Argon document")
        argon_version = d["OpenCMISS-Argon Version"]
        if argon_version > mainsettings.VERSION_LIST:
            raise ArgonError("Document version is greater than this version of Argon (" + mainsettings.VERSION_STRING + "). Please update your Argon application.")
        # Ideally would enclose following in:
        # try: zincRegion.beginHierarchicalChange() ... finally: zincRegion.endHierarchicalChange()
        # Can't do this due to Zinc issue 3924 which prevents computed field wrappers being created, so graphics can't find fields
        if "Tessellations" in d:
            self._tessellations.deserialize(d["Tessellations"])
        if "Spectrums" in d:
            self._spectrums.deserialize(d["Spectrums"])
        if "Materials" in d:
            self._materials.deserialize(d["Materials"])
        if "Views" in d:
            self._view_manager.deserialize(d["Views"])
        self._rootRegion.deserialize(d["RootRegion"])

    def serialize(self, basePath=None):
        dictOutput = {
            "OpenCMISS-Argon Version": mainsettings.VERSION_LIST,
            "Spectrums": self._spectrums.serialize(),
            "Materials": self._materials.serialize(),
            "Views": self._view_manager.serialize(),
            "Tessellations": self._tessellations.serialize(),
            "RootRegion": self._rootRegion.serialize(basePath)
        }
        return json.dumps(dictOutput, default=lambda o: o.__dict__, sort_keys=True, indent=2)

    def getZincContext(self):
        return self._zincContext

    def getRootRegion(self):
        return self._rootRegion

    def getSpectrums(self):
        return self._spectrums

    def getMaterials(self):
        return self._materials

    def getViewManager(self):
        return self._view_manager

    def getTessellations(self):
        return self._tessellations

    def findRegion(self, name):
        if not name.endswith(REGION_PATH_SEPARATOR):
            name += REGION_PATH_SEPARATOR
        return _findSubRegion(self._rootRegion, name)


def _findSubRegion(region, name):
    region_path = region.getPath()
    if name == region_path:
        return region
    elif name.startswith(region_path):
        for child_index in range(region.getChildCount()):
            result = _findSubRegion(region.getChild(child_index), name)
            if result is not None:
                return result

    return None



"""
   Copyright 2016 University of Auckland

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
"""
import json

from opencmiss.zinc.status import OK as ZINC_OK
from opencmiss.argon.argonerror import ArgonError


class ArgonTessellations(object):
    """
    Manages and serializes Zinc Tessellations within Neon.
    """

    def __init__(self, zincContext):
        self._zincContext = zincContext
        self._tessellationmodule = zincContext.getTessellationmodule()

    def getZincContext(self):
        return self._zincContext

    def deserialize(self, dictInput):
        tessellationsDescription = json.dumps(dictInput)
        result = self._tessellationmodule.readDescription(tessellationsDescription)
        if result != ZINC_OK:
             raise ArgonError("Failed to read tessellations")

    def serialize(self):
        tessellationsDescription = self._tessellationmodule.writeDescription()
        dictOutput = json.loads(tessellationsDescription)
        return dictOutput



"""
   Copyright 2015 University of Auckland

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
"""
import os

from opencmiss.zinc.streamregion import StreaminformationRegion
from opencmiss.argon.argonerror import ArgonError


def fileNameToRelativePath(fileName, basePath):
    if (basePath is None) or (not os.path.isabs(fileName)) or (os.path.commonprefix([fileName, basePath]) == ""):
        return fileName
    return os.path.relpath(fileName, basePath)


class ArgonModelSourceFile(object):

    def __init__(self, fileName=None, dictInput=None):
        self._time = None
        self._format = None
        self._edit = False
        self._region_name = None
        self._loaded = False
        if fileName is not None:
            self._fileName = fileName
        else:
            self._deserialize(dictInput)

    def getType(self):
        return "FILE"

    def addToZincStreaminformationRegion(self, streamInfo):
        if self._edit:
            return
        if not self._fileName:
            self._edit = True
            return
        resource = streamInfo.createStreamresourceFile(self._fileName)
        self._loaded = True
        if self._time is not None:
            streamInfo.setResourceAttributeReal(resource, StreaminformationRegion.ATTRIBUTE_TIME, self._time)
        # if self._format is not None:
        #    if format == "EX":
        #        #can't set per-resource file format
        #        #streamInfo.setResourceFileFormat(resource, StreaminformationRegion.FILE_FORMAT_EX)

    def unloaded(self):
        self._loaded = False

    def getFileName(self):
        return self._fileName

    def setFileName(self, fileName):
        self._fileName = fileName

    def getRegionName(self):
        return self._region_name

    def setRegionName(self, region_name):
        self._region_name = region_name

    def getTime(self):
        return self._time

    def setTime(self, time):
        self._time = time

    def getDisplayName(self):
        editText = "[To Apply] " if self._edit else ""
        if self._time is None:
            timeText = ""
        else:
            timeText = ", time " + repr(self._time)
        displayFileName = os.path.basename(self._fileName)
        return editText + "File " + displayFileName + timeText

    def isLoaded(self):
        return self._loaded

    def isEdit(self):
        return self._edit

    def setEdit(self, edit):
        self._edit = edit

    def _deserialize(self, dictInput):
        # convert to absolute file path so can save Neon file to new location and get correct relative path
        self._fileName = os.path.abspath(dictInput["FileName"])
        if "Time" in dictInput:
            self._time = dictInput["Time"]
        if "Format" in dictInput:
            self._format = dictInput["Format"]
        if "Edit" in dictInput:
            self._edit = dictInput["Edit"]
        if "RegionName" in dictInput:
            self._region_name = dictInput["RegionName"]

    def serialize(self, basePath=None):
        dictOutput = {}
        dictOutput["Type"] = self.getType()
        dictOutput["FileName"] = fileNameToRelativePath(self._fileName, basePath)
        if self._region_name is not None:
            dictOutput["RegionName"] = self._region_name
        if self._time is not None:
            dictOutput["Time"] = self._time
        if self._edit:
            dictOutput["Edit"] = True
        return dictOutput

def deserializeArgonModelSource(dictInput):
    """
    Factory method for creating the appropriate neon model source type from the dict serialization
    """
    if "Type" not in dictInput:
        raise ArgonError("Model source is missing Type attribute")
    modelSource = None
    typeString = dictInput["Type"]
    if typeString == "FILE":
        modelSource = ArgonModelSourceFile(dictInput=dictInput)
    else:
        raise ArgonError("Model source has unrecognised Type " + typeString)
    return modelSource



"""
   Copyright 2015 University of Auckland

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
"""
import json

from opencmiss.argon.argonmodelsources import deserializeArgonModelSource
from opencmiss.argon.argonerror import ArgonError
from opencmiss.zinc.status import OK as ZINC_OK


REGION_PATH_SEPARATOR = "/"


class ArgonRegion(object):

    def __init__(self, name, zincRegion, parent=None):
        self._name = name
        self._parent = parent
        self._children = []
        self._modelSources = []
        self._zincRegion = zincRegion
        # record whether region was created by ancestor model source; see: _reloadModelSources
        self._ancestorModelSourceCreated = False
        # callback class, only for root region
        if not parent:
            self._regionChangeCallbacks = []
        
        self._fieldTypeDict = {}

    # def __del__(self):
    #    print("NeonRegion.__del__ " + self.getDisplayName())

    def freeContents(self):
        """
        Deletes subobjects of region to help free memory held by Zinc objects earlier.
        """
        del self._zincRegion
        for child in self._children:
            child.freeContents()

    def _createBlankCopy(self):
        zincRegion = self._zincRegion.createRegion()
        if self._name:
            zincRegion.setName(self._name)
        blankRegion = ArgonRegion(self._name, zincRegion, self._parent)
        return blankRegion

    def _assign(self, source):
        """
        Replace contents of self with that of source. Fixes up Zinc parent/child region relationships.
        """
        if self._parent:
            oldZincRegion = self._zincRegion
            zincSiblingAfter = oldZincRegion.getNextSibling()
        else:
            oldZincRegion = None
            zincSiblingAfter = None
        self.freeContents()
        self._name = source._name
        # self._parent = source._parent should not be changed
        self._children = source._children
        for child in self._children:
            child._parent = self
        self._modelSources = source._modelSources
        self._zincRegion = source._zincRegion
        # self._ancestorModelSourceCreated is unchanged
        if self._parent:
            self._parent._zincRegion.removeChild(oldZincRegion)
            self._parent._zincRegion.insertChildBefore(self._zincRegion, zincSiblingAfter)

    def _informRegionChange(self, treeChange):
        """
        Called by regions when their tree structure changes or zinc regions are rebuilt.
        Informs registered clients of change. Root region handle these signals for whole tree.
        """
        rootRegion = self
        while rootRegion._parent:
            rootRegion = rootRegion._parent
        for callback in rootRegion._regionChangeCallbacks:
            callback(self, treeChange)

    def connectRegionChange(self, callableObject):
        """
        Request callbacks on region tree changes.
        :param callableObject: Callable object taking a NeonRegion argument and a boolean flag which is True if tree
        structure below region needs to be rebuilt.
        """
        self._regionChangeCallbacks.append(callableObject)

    def _loadModelSourceStreams(self, streamInfo):
        self._zincRegion.beginHierarchicalChange()
        result = self._zincRegion.read(streamInfo)
        fieldmodule = self._zincRegion.getFieldmodule()
        fieldmodule.defineAllFaces()
        self._zincRegion.endHierarchicalChange()
        if result != ZINC_OK:
            raise ArgonError("Failed to load model sources into region " + self.getPath())

    def _loadModelSource(self, modelSource):
        streamInfo = self._zincRegion.createStreaminformationRegion()
        modelSource.addToZincStreaminformationRegion(streamInfo)
        self._loadModelSourceStreams(streamInfo)
        newRegionCount = self._discoverNewZincRegions()
        self._informRegionChange(newRegionCount > 0)

    def _loadModelSources(self):
        streamInfo = self._zincRegion.createStreaminformationRegion()
        for modelSource in self._modelSources:
            modelSource.addToZincStreaminformationRegion(streamInfo)
        self._loadModelSourceStreams(streamInfo)

    def _reload(self):
        """
        Must be called when already-loaded model source modified or deleted.
        Saves and reloads region tree, starting at ancestor if this region was created by its model source.
        """
        if self._ancestorModelSourceCreated:
            self._parent._reload()
        else:
            # beware this breaks parent/child links such as current selection / hierarchical groups
            dictSave = self.serialize()
            tmpRegion = self._createBlankCopy()
            tmpRegion.deserialize(dictSave)
            self._assign(tmpRegion)
            self._informRegionChange(True)

    def _discoverNewZincRegions(self):
        """
        Ensure there are Neon regions for every Zinc Region in tree
        :return: Number of new descendant regions created
        """
        newRegionCount = 0
        zincChildRef = self._zincRegion.getFirstChild()
        while zincChildRef.isValid():
            childName = zincChildRef.getName()
            neonChild = self._findChildByName(childName)
            if not neonChild:
                neonChild = ArgonRegion(childName, zincChildRef, self)
                neonChild._ancestorModelSourceCreated = True
                self._children.append(neonChild)
                newRegionCount += (1 + neonChild._discoverNewZincRegions())
            zincChildRef = zincChildRef.getNextSibling()
        return newRegionCount

    def _findChildByName(self, name):
        for child in self._children:
            if child._name == name:
                return child
        return None

    def _generateChildName(self):
        count = len(self._children) + 1
        while True:
            name = "region" + str(count)
            if not self._findChildByName(name):
                return name
            count += 1
        return None

    def deserialize(self, dictInput):
        if "Model" in dictInput:
            model = dictInput["Model"]
            if "Sources" in model:
                try:
                    for dictModelSource in model["Sources"]:
                        modelSource = deserializeArgonModelSource(dictModelSource)
                        if modelSource:
                            self._modelSources.append(modelSource)
                except ArgonError as neonError:
                    raise ArgonError(neonError.getMessage() + " in region " + self.getPath())
                self._loadModelSources()

        if "Fieldmodule" in dictInput:
            # must define fields before scene otherwise referenced fields won't exist
            fieldmodule = self._zincRegion.getFieldmodule()
            fieldmoduleDescription = json.dumps(dictInput["Fieldmodule"])
            result = fieldmodule.readDescription(fieldmoduleDescription)
            if result != ZINC_OK:
                 raise ArgonError("Failed to read field module description into region " + self.getPath())

        if "Scene" in dictInput:
            scene = self._zincRegion.getScene()
            sceneDescription = json.dumps(dictInput["Scene"])
            result = scene.readDescription(sceneDescription, True)
            if result != ZINC_OK:
                 raise ArgonError("Failed to read scene description into region " + self.getPath())

        if ("Fieldmodule" in dictInput) and isinstance(dictInput["Fieldmodule"], dict) and \
                ("Fields" in dictInput["Fieldmodule"]):
            # clear IsManaged flags for fields so marked; do last otherwise fields in use by scene may be destroyed
            fieldsDict = dictInput["Fieldmodule"]["Fields"]
            for fieldDict in fieldsDict:
                isManaged = fieldDict["IsManaged"]
                if not isManaged:
                    field = fieldmodule.findFieldByName(fieldDict["Name"])
                    if field.isValid():
                        field.setManaged(False)
                for currentKey in fieldDict.keys():
                    if currentKey.find('Field') != -1:
                        self._fieldTypeDict[fieldDict["Name"]] = currentKey

        # following assumes no neon child regions exist, i.e. we are deserializing into a blank region
        # for each neon region, ensure there is a matching zinc region in the same order, and recurse
        zincChildRef = self._zincRegion.getFirstChild()
        if "ChildRegions" in dictInput:
            for dictChild in dictInput["ChildRegions"]:
                childName = dictChild["Name"]
                # see if zinc child with this name created by model source here or in ancestor region
                ancestorModelSourceCreated = True
                zincChild = self._zincRegion.findChildByName(childName)
                if zincChildRef.isValid() and (zincChild == zincChildRef):
                    zincChildRef = zincChildRef.getNextSibling()
                else:
                    if not zincChild.isValid():
                        zincChild = self._zincRegion.createRegion()
                        zincChild.setName(childName)
                        ancestorModelSourceCreated = False
                    self._zincRegion.insertChildBefore(zincChild, zincChildRef)
                neonChild = ArgonRegion(childName, zincChild, self)
                neonChild._ancestorModelSourceCreated = ancestorModelSourceCreated
                self._children.append(neonChild)
                neonChild.deserialize(dictChild)
        self._discoverNewZincRegions()

    def serialize(self, basePath=None):
        dictOutput = {}
        if self._name:
            dictOutput["Name"] = self._name
        dictOutput["Model"] = {}
        if self._modelSources:
            tmpOutput = []
            for modelSource in self._modelSources:
                tmpOutput.append(modelSource.serialize(basePath))
            dictOutput["Model"]["Sources"] = tmpOutput
        if not dictOutput["Model"]:
            dictOutput.pop("Model")
        if self._zincRegion:
            fieldmodule = self._zincRegion.getFieldmodule()
            fieldmoduleDescription = fieldmodule.writeDescription()
            dictOutput["Fieldmodule"] = json.loads(fieldmoduleDescription)

            scene = self._zincRegion.getScene()
            sceneDescription = scene.writeDescription()
            dictOutput["Scene"] = json.loads(sceneDescription)
        if self._children:
            tmpOutput = []
            for child in self._children:
                tmpOutput.append(child.serialize(basePath))
            dictOutput["ChildRegions"] = tmpOutput
        return dictOutput

    def getDisplayName(self):
        if self._name:
            return self._name
        elif not self._parent:
            return REGION_PATH_SEPARATOR
        return "?"

    def getName(self):
        return self._name

    def getPath(self):
        if self._name:
            return self._parent.getPath() + self._name + REGION_PATH_SEPARATOR
        return REGION_PATH_SEPARATOR

    def getParent(self):
        return self._parent

    def getZincRegion(self):
        return self._zincRegion

    def getChildCount(self):
        return len(self._children)

    def getChild(self, index):
        return self._children[index]
    
    def getFieldTypeDict(self):
        return self._fieldTypeDict
    
    def addFieldTypeToDict(self, field, fieldType):
        if field and field.isValid():
            self._fieldTypeDict[field.getName()] = fieldType
            
    def replaceFieldTypeKey(self, oldName, newName):
        if oldName in self._fieldTypeDict:
            self._fieldTypeDict[newName] = self._fieldTypeDict.pop(oldName)
    
    def clear(self):
        """
        Clear all contents of region. Can be called for root region
        """
        tmpRegion = self._createBlankCopy()
        self._assign(tmpRegion)
        if self._ancestorModelSourceCreated:
            self._reload()
        else:
            self._informRegionChange(True)

    def createChild(self):
        """
        Create a child region with a default name
        :return: The new Neon Region
        """
        childName = self._generateChildName()
        zincRegion = self._zincRegion.createChild(childName)
        if zincRegion.isValid():
            childRegion = ArgonRegion(childName, zincRegion, self)
            self._children.append(childRegion)
            self._informRegionChange(True)
            return childRegion
        return None

    def removeChild(self, childRegion):
        """
        Remove child region and destroy
        """
        self._children.remove(childRegion)
        self._zincRegion.removeChild(childRegion._zincRegion)
        childRegion._parent = None
        childRegion.freeContents()
        if childRegion._ancestorModelSourceCreated:
            self._reload()
        else:
            self._informRegionChange(True)

    def remove(self):
        """
        Remove self from region tree and destroy; replace with blank region if root
        """
        if self._parent:
            self._parent.removeChild(self)
        else:
            self.clear()

    def setName(self, name):
        if not self._parent:
            return False
        if len(name) == 0:
            return False
        if self._ancestorModelSourceCreated:
            return False
        if ZINC_OK != self._zincRegion.setName(name):
            return False
        self._name = name
        self._informRegionChange(True)
        return True

    def getModelSources(self):
        return self._modelSources

    def addModelSource(self, modelSource):
        """
        Add model source, applying it if not currently editing
        :param modelSource: The model source to add
        """
        self._modelSources.append(modelSource)
        if not modelSource.isEdit():
            self.applyModelSource(modelSource)

    def applyModelSource(self, modelSource):
        """
        Apply model source, loading it or reloading it with all other sources as required
        :param modelSource: The model source to apply
        """
        modelSource.setEdit(False)
        if modelSource.isLoaded():
            self._reload()
        else:
            self._loadModelSource(modelSource)

    def removeModelSource(self, modelSource):
        """
        Remove model source, reloading model if it removed source had been loaded
        :param modelSource: The model source to remove
        """
        self._modelSources.remove(modelSource)
        if modelSource.isLoaded():
            modelSource.unloaded()
            self._reload()



"""
   Copyright 2015 University of Auckland

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
"""

import sys
import logging

try:
    from PySide2 import QtCore
    HAVE_PYSIDE2 = True
except ImportError:
    HAVE_PYSIDE2 = False

from opencmiss.zinc.logger import Logger

ENABLE_STD_STREAM_CAPTURE = False


if HAVE_PYSIDE2:
    class CustomStreamImpl(QtCore.QObject):
        # Signal is a class variable; PySide creates per-instance SignalInstance object of same name
        messageWritten = QtCore.Signal(str, str)

        # Note: if implementing __init__ you must call super __init__ for Signals to work.
        # def __init__(self):
        #     super(CustomStreamImpl, self).__init__()

        def flush(self):
            pass

        def fileno(self):
            return -1

        def write(self, msg, level="INFORMATION"):
            if not self.signalsBlocked():
                self.messageWritten.emit(msg, level)
else:
    class CustomStreamImpl(object):

        def flush(self):
            pass

        def fileno(self):
            return -1

        def write(self, msg, level="INFORMATION"):
            sys.stdout.write(f'{level} - {msg}')


class CustomStream(object):
    _stdout = None
    _stderr = None

    @staticmethod
    def stdout():
        if CustomStream._stdout is None:
            CustomStream._stdout = CustomStreamImpl()
            if ENABLE_STD_STREAM_CAPTURE:
                sys.stdout = CustomStream._stdout
        return CustomStream._stdout

    @staticmethod
    def stderr():
        if CustomStream._stderr is None:
            CustomStream._stderr = CustomStreamImpl()
            if ENABLE_STD_STREAM_CAPTURE:
                sys.stderr = CustomStream._stderr
        return CustomStream._stderr


class LogsToWidgetHandler(logging.Handler):

    def __init__(self):
        logging.Handler.__init__(self)

    def emit(self, record):
        levelString = record.levelname
        record = self.format(record)
        if record:
            CustomStream.stdout().write('%s\n' % record, levelString)


def setup_custom_logger(name, callback):

    formatter = logging.Formatter(fmt='%(asctime)s - %(levelname)s - %(module)s - %(message)s')

    handler = LogsToWidgetHandler()
    handler.setFormatter(formatter)

    argonLogger = logging.getLogger(name)
    argonLogger.setLevel(logging.DEBUG)
    argonLogger.addHandler(handler)
    return argonLogger


class ArgonLogger(object):
    _logger = None
    _zincLogger = None
    _loggerNotifier = None
    _callback = None

    @staticmethod
    def setCallback(callback):
        ArgonLogger._callback = callback

    @staticmethod
    def getLogger():
        if not ArgonLogger._logger:
            ArgonLogger._logger = setup_custom_logger("Argon", ArgonLogger._callback)
        return ArgonLogger._logger

    @staticmethod
    def closeLogger():
        if ArgonLogger._logger:
            del ArgonLogger._logger
            ArgonLogger._logger = None
            sys.stdout = sys.__stdout__
            sys.stderr = sys.__stderr__

    @staticmethod
    def writeErrorMessage(string):
        ArgonLogger.getLogger().error(string)

    @staticmethod
    def writeWarningMessage(string):
        ArgonLogger.getLogger().warning(string)

    @staticmethod
    def writeInformationMessage(string):
        ArgonLogger.getLogger().info(string)

    @staticmethod
    def loggerCallback(event):
        if event.getChangeFlags() == Logger.CHANGE_FLAG_NEW_MESSAGE:
            text = event.getMessageText()
            if event.getMessageType() == Logger.MESSAGE_TYPE_ERROR:
                ArgonLogger.writeErrorMessage(text)
            elif event.getMessageType() == Logger.MESSAGE_TYPE_WARNING:
                ArgonLogger.writeWarningMessage(text)
            elif event.getMessageType() == Logger.MESSAGE_TYPE_INFORMATION:
                ArgonLogger.writeInformationMessage(text)

    @staticmethod
    def setZincContext(zincContext):
        if ArgonLogger._loggerNotifier:
            ArgonLogger._loggerNotifier.clearCallback()
        ArgonLogger._zincLogger = zincContext.getLogger()
        ArgonLogger._loggerNotifier = ArgonLogger._zincLogger.createLoggernotifier()
        ArgonLogger._loggerNotifier.setCallback(ArgonLogger.loggerCallback)
